@@ -13,6 +13,22 @@ from threading import Thread
 from Queue import Queue
 
 
+class ExtendableFunction(object):
+    """A special type of class that implements subclassable functions"""
+    def __new__(cls, *args, **kwargs):
+        self = object.__new__(cls)
+        return cls.execute(self, *args, **kwargs)
+
+    def execute(self, *args, **kwargs):
+        """This method is what get's called when the class is "called".
+
+        By default it returns None.  Think of it as the actual function
+        definition itself.  When creating extendable functions this is the
+        method that should be overridden.
+        """
+        return None
+
+
 def job_wrapper(job_function, resultsqueue, *args, **kwargs):
     """sticks the results of the function (or error info) into a queue"""
     final_result = return_result_or_error(job_function, *args, **kwargs)
