@@ -9,7 +9,99 @@ import os
 import sys
 import unittest
 import copy
-from philutils.containers import DotDict, dictapply
+import threading
+from time import sleep
+from Queue import Queue
+from philutils.containers import DotDict, dictapply, ThreadSafeDotDict
+from philutils.tests.test_reverse_semaphore import queue2list
+
+
+class ThreadTester(object):
+    def __init__(self, dotdict, result_queue):
+        self.dotdict = dotdict
+        self.result_queue = result_queue
+        self.finished = threading.Event()
+
+    def start(self, target, *args, **kwargs):
+        args2send = [target, self.dotdict]
+        args2send.extend(args)
+        self.thread = threading.Thread(
+            target=self.main,
+            args=args2send,
+            kwargs=kwargs
+        )
+        self.thread.daemon = True
+        self.daemon = True
+        self.thread.start()
+
+    def main(self, target,  *args, **kwargs):
+        self.result_queue.put(target(*args, **kwargs))
+        self.finished.set()
+
+
+class TestProtectedDotDict(unittest.TestCase):
+    def setUp(self):
+        self.valuequeue = Queue()
+        self.dotdict = ThreadSafeDotDict()
+        self.dotdict.animal = 'monkey'
+        self.dotdict.number = 444
+        self.dotdict.child = DotDict()
+        self.dotdict.child.animal = 'cow'
+        self.dotdict.child.number = 2
+        self.tester = ThreadTester(self.dotdict, self.valuequeue)
+
+    def test_lock_utilized_when_getting_attr(self):
+        args = ('animal', )
+        themethod = ThreadSafeDotDict.__getattr__
+        with self.dotdict.lock:
+            self.tester.start(themethod, *args)
+            sleep(.5)
+            self.valuequeue.put('starting')
+        self.tester.finished.wait(5)
+        results = queue2list(self.valuequeue)
+        expected = ['starting', 'monkey']
+        self.assertEqual(results, expected)
+
+    def test_lock_utilized_when_setting_attr(self):
+        args = ('bug', 'ant')
+        themethod = ThreadSafeDotDict.__setattr__
+        with self.dotdict.lock:
+            self.tester.start(themethod, *args)
+            sleep(.5)
+            self.assertEqual(len(self.dotdict.keys()), 3)
+        self.tester.finished.wait(5)
+        self.assertEqual(len(self.dotdict.keys()), 4)
+
+    def test_lock_utilized_when_deling_attr(self):
+        args = ('animal',)
+        themethod = ThreadSafeDotDict.__delattr__
+        with self.dotdict.lock:
+            self.tester.start(themethod, *args)
+            sleep(.5)
+            self.assertEqual(len(self.dotdict.keys()), 3)
+        self.tester.finished.wait(5)
+        self.assertEqual(len(self.dotdict.keys()), 2)
+
+    def test_lock_utilized_when_getting_dict_keys(self):
+        del self.dotdict.child
+
+        def method(somedict):
+            values = set()
+            for key in somedict.keys():
+                values.add(somedict.pop(key))
+            return values
+
+        args = []
+        with self.dotdict.lock:
+            self.tester.start(method, *args)
+            sleep(.5)
+            tempresults = queue2list(self.valuequeue)
+            self.assertEqual(tempresults, [])
+            self.dotdict.bug = 'ant'
+        self.tester.finished.wait(5)
+        results = queue2list(self.valuequeue)
+        expected = [set(['ant', 'monkey', 444])]
+        self.assertEqual(results, expected)
 
 
 class TestDotDict(unittest.TestCase):
@@ -141,3 +233,6 @@ class Test_dictapply(unittest.TestCase):
         )
         result = dictapply(base, overrider)
         self.assertEqual(result, expected)
+
+if __name__ == '__main__':
+    unittest.main()
