@@ -11,6 +11,32 @@ import unittest
 from philutils import execution
 
 
+class get_odds_less_than(execution.ExtendableFunction):
+    modulo = 1
+
+    def execute(self, n):
+        return [x for x in range(n) if self.needs_to_be_added(x)]
+
+    def needs_to_be_added(self, x):
+        return x % 2 == self.modulo
+
+
+class get_evens_less_than(get_odds_less_than):
+    modulo = 0
+
+
+class alt_get_evens_less_than(get_odds_less_than):
+    def needs_to_be_added(self, x):
+        return not get_odds_less_than.needs_to_be_added(self, x)
+
+
+class alt_get_odds_less_than(alt_get_evens_less_than):
+    modulo = 1
+
+    def needs_to_be_added(self, x):
+        return not alt_get_evens_less_than.needs_to_be_added(self, x)
+
+
 def multiply(first=1, second=0):
     return first * second
 
@@ -39,6 +65,31 @@ def raises(x):
     raise Exception("the value x is '%s'" % x)
 
 
+class TestExtendableFunctions(unittest.TestCase):
+    def test_extendable_function_base_class_returns_none(self):
+        self.assertEqual(execution.ExtendableFunction(), None)
+
+    def test_extendable_function_is_executed_correctly(self):
+        result = get_odds_less_than(20)
+        expected = range(1, 20, 2)
+        self.assertEqual(result, expected)
+
+    def test_that_inheritance_works(self):
+        result = get_evens_less_than(30)
+        expected = range(0, 30, 2)
+        self.assertEqual(result, expected)
+
+    def test_that_instance_properties_are_used_when_inheriting(self):
+        result = alt_get_odds_less_than(30)
+        expected = range(1, 30, 2)
+        self.assertEqual(result, expected)
+
+    def test_that_supers_work(self):
+        result = alt_get_evens_less_than(30)
+        expected = range(0, 30, 2)
+        self.assertEqual(result, expected)
+
+
 class Test_retun_results_or_error(unittest.TestCase):
     def test_that_sucessful_run_with_no_arguments_has_status(self):
         result = execution.return_result_or_error(three)
